import {
  Controller,
  Get,
  Post,
  Body,
  Param,
  Delete,
  Put,
  UseGuards,
  Query,
  HttpCode,
  HttpStatus,
} from '@nestjs/common';
import { GroupsService } from '../services/groups.service';
import { CreateGroupDto } from '../dto/create-group.dto';
import { UpdateGroupDto } from '../dto/update-group.dto';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';

@Controller('groups')
@UseGuards(JwtAuthGuard)
export class GroupsController {
  constructor(private readonly groupsService: GroupsService) {}

  /**
   * Create a new group
   */
  @Post()
  create(@Body() createGroupDto: CreateGroupDto) {
    return this.groupsService.create(createGroupDto);
  }

  /**
   * Get all groups or filter by project ID
   */
  @Get()
  findAll(@Query('projectId') projectId?: string) {
    if (projectId) {
      return this.groupsService.findByProjectId(projectId);
    }
    return this.groupsService.findAll();
  }

  /**
   * Get a group by ID
   */
  @Get(':id')
  findOne(@Param('id') id: string) {
    return this.groupsService.findById(id);
  }

  /**
   * Update a group
   */
  @Put(':id')
  update(@Param('id') id: string, @Body() updateGroupDto: UpdateGroupDto) {
    return this.groupsService.update(id, updateGroupDto);
  }

  /**
   * Delete a group
   */
  @Delete(':id')
  remove(@Param('id') id: string) {
    return this.groupsService.remove(id);
  }

  /**
   * Add a person to a group
   */
  @Post(':id/persons/:personId')
  @HttpCode(HttpStatus.CREATED)
  addPersonToGroup(
    @Param('id') groupId: string,
    @Param('personId') personId: string,
  ) {
    return this.groupsService.addPersonToGroup(groupId, personId);
  }

  /**
   * Remove a person from a group
   */
  @Delete(':id/persons/:personId')
  removePersonFromGroup(
    @Param('id') groupId: string,
    @Param('personId') personId: string,
  ) {
    return this.groupsService.removePersonFromGroup(groupId, personId);
  }

  /**
   * Get all persons in a group
   */
  @Get(':id/persons')
  getPersonsInGroup(@Param('id') groupId: string) {
    return this.groupsService.getPersonsInGroup(groupId);
  }
}
