"use client";

import { useState } from "react";
import { useRouter } from "next/navigation";
import Link from "next/link";
import { useForm, Controller } from "react-hook-form";
import { Button } from "@/components/ui/button";
import { Card, CardContent, CardDescription, CardFooter, CardHeader, CardTitle } from "@/components/ui/card";
import { Input } from "@/components/ui/input";
import { Label } from "@/components/ui/label";
import { Textarea } from "@/components/ui/textarea";
import { 
  ArrowLeft,
  Loader2,
  Save,
  CircleDot
} from "lucide-react";
import { Badge } from "@/components/ui/badge";
import { toast } from "sonner";
import {
  Select,
  SelectContent,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from "@/components/ui/select";

// Type definitions
interface TagFormData {
  name: string;
  description: string;
  color: string;
}

// Available colors
const colors = [
  { name: "Bleu", value: "blue" },
  { name: "Vert", value: "green" },
  { name: "Violet", value: "purple" },
  { name: "Rose", value: "pink" },
  { name: "Orange", value: "orange" },
  { name: "Jaune", value: "yellow" },
  { name: "Ambre", value: "amber" },
  { name: "Rouge", value: "red" },
];

// Map color names to Tailwind classes
const colorMap: Record<string, string> = {
  blue: "bg-blue-100 text-blue-800 dark:bg-blue-900 dark:text-blue-300",
  green: "bg-green-100 text-green-800 dark:bg-green-900 dark:text-green-300",
  purple: "bg-purple-100 text-purple-800 dark:bg-purple-900 dark:text-purple-300",
  pink: "bg-pink-100 text-pink-800 dark:bg-pink-900 dark:text-pink-300",
  orange: "bg-orange-100 text-orange-800 dark:bg-orange-900 dark:text-orange-300",
  yellow: "bg-yellow-100 text-yellow-800 dark:bg-yellow-900 dark:text-yellow-300",
  amber: "bg-amber-100 text-amber-800 dark:bg-amber-900 dark:text-amber-300",
  red: "bg-red-100 text-red-800 dark:bg-red-900 dark:text-red-300",
};

export default function NewTagPage() {
  const router = useRouter();
  const [isSubmitting, setIsSubmitting] = useState(false);
  
  const { register, handleSubmit, control, watch, formState: { errors } } = useForm<TagFormData>({
    defaultValues: {
      name: "",
      description: "",
      color: "blue"
    }
  });

  const selectedColor = watch("color");

  const onSubmit = async (data: TagFormData) => {
    setIsSubmitting(true);
    try {
      // In a real app, this would be an API call to create a new tag
      await new Promise(resolve => setTimeout(resolve, 1000));
      
      // Simulate a successful response with a tag ID
      const tagId = Date.now();
      
      toast.success("Tag créé avec succès");
      router.push("/tags");
    } catch (error) {
      console.error("Error creating tag:", error);
      toast.error("Erreur lors de la création du tag");
    } finally {
      setIsSubmitting(false);
    }
  };

  return (
    <div className="flex flex-col gap-6">
      <div className="flex items-center gap-2">
        <Button variant="outline" size="icon" asChild>
          <Link href="/tags">
            <ArrowLeft className="h-4 w-4" />
          </Link>
        </Button>
        <h1 className="text-3xl font-bold">Nouveau tag</h1>
      </div>

      <Card>
        <form onSubmit={handleSubmit(onSubmit)}>
          <CardHeader>
            <CardTitle>Informations du tag</CardTitle>
            <CardDescription>
              Créez un nouveau tag pour catégoriser les personnes
            </CardDescription>
          </CardHeader>
          <CardContent className="space-y-4">
            <div className="space-y-2">
              <Label htmlFor="name">Nom du tag</Label>
              <Input
                id="name"
                placeholder="Ex: Frontend"
                {...register("name", { 
                  required: "Le nom du tag est requis",
                  minLength: {
                    value: 2,
                    message: "Le nom doit contenir au moins 2 caractères"
                  }
                })}
              />
              {errors.name && (
                <p className="text-sm text-destructive">{errors.name.message}</p>
              )}
            </div>

            <div className="space-y-2">
              <Label htmlFor="description">Description</Label>
              <Textarea
                id="description"
                placeholder="Ex: Développement frontend"
                rows={3}
                {...register("description", { 
                  required: "La description du tag est requise",
                  minLength: {
                    value: 5,
                    message: "La description doit contenir au moins 5 caractères"
                  }
                })}
              />
              {errors.description && (
                <p className="text-sm text-destructive">{errors.description.message}</p>
              )}
            </div>

            <div className="space-y-2">
              <Label htmlFor="color">Couleur</Label>
              <Controller
                name="color"
                control={control}
                rules={{ required: "La couleur est requise" }}
                render={({ field }) => (
                  <Select
                    onValueChange={field.onChange}
                    defaultValue={field.value}
                  >
                    <SelectTrigger>
                      <SelectValue placeholder="Sélectionnez une couleur" />
                    </SelectTrigger>
                    <SelectContent>
                      {colors.map((color) => (
                        <SelectItem key={color.value} value={color.value}>
                          <div className="flex items-center">
                            <CircleDot className={`mr-2 h-4 w-4 text-${color.value}-500`} />
                            {color.name}
                          </div>
                        </SelectItem>
                      ))}
                    </SelectContent>
                  </Select>
                )}
              />
              {errors.color && (
                <p className="text-sm text-destructive">{errors.color.message}</p>
              )}
            </div>

            <div className="space-y-2">
              <Label>Aperçu</Label>
              <div className="flex items-center gap-2">
                <Badge className={colorMap[selectedColor]}>
                  {watch("name") || "Nom du tag"}
                </Badge>
                <span className="text-sm text-muted-foreground">
                  {watch("description") || "Description du tag"}
                </span>
              </div>
            </div>
          </CardContent>
          <CardFooter className="flex justify-between">
            <Button variant="outline" asChild>
              <Link href="/tags">Annuler</Link>
            </Button>
            <Button type="submit" disabled={isSubmitting}>
              {isSubmitting ? (
                <>
                  <Loader2 className="mr-2 h-4 w-4 animate-spin" />
                  Création en cours...
                </>
              ) : (
                <>
                  <Save className="mr-2 h-4 w-4" />
                  Créer le tag
                </>
              )}
            </Button>
          </CardFooter>
        </form>
      </Card>
    </div>
  );
}